# Copyright (c) 2012, Sven Thiele <sthiele78@gmail.com>
#
# This file is part of meneco.
#
# meneco is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# meneco is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with meneco.  If not, see <http://www.gnu.org/licenses/>.

import os
import tempfile


def clean_up():
    if os.path.isfile("parser.out"):
        os.remove("parser.out")
    if os.path.isfile("parsetab.py"):
        os.remove("parsetab.py")
    if os.path.isfile("asp_py_lextab.py"):
        os.remove("asp_py_lextab.py")
    if os.path.isfile("asp_py_lextab.pyc"):
        os.remove("asp_py_lextab.pyc")
    if os.path.isfile("asp_py_parsetab.py"):
        os.remove("asp_py_parsetab.py")
    if os.path.isfile("asp_py_parsetab.pyc"):
        os.remove("asp_py_parsetab.pyc")


def to_file(termset, outputfile=None):
    """write (append) the content of the TermSet into a file

    Args:
        termset (TermSet): ASP termset
        outputfile (str, optional): Defaults to None. name of the output file
    """
    if outputfile:
        f = open(outputfile, "a")
    else:
        fd, outputfile = tempfile.mkstemp(suffix=".lp", prefix="meneco_")
        f = os.fdopen(fd, "a")
    for t in termset:
        f.write(str(t) + ".\n")
    f.close()
    return outputfile



import logging
import sys

from meneco.meneco import cmd_meneco

logging.basicConfig(format="%(message)s", level=logging.DEBUG)
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)


def main_meneco(args=None):
    cmd_meneco(sys.argv[1:])


if __name__ == "__main__":
    main_meneco()



# Copyright (c) 2014, Sven Thiele <sthiele78@gmail.com>
#
# This file is part of meneco.
#
# meneco is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# meneco is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with meneco.  If not, see <http://www.gnu.org/licenses/>.

import os

import clyngor
from clyngor.as_pyasp import Atom, TermSet

from meneco import utils

root = __file__.rsplit("/", 1)[0]
unproducible_prg = root + "/encodings/unproducible_targets.lp"
ireaction_prg = root + "/encodings/ireactions.lp"
minimal_completion_prg = root + "/encodings/card_min_completions_all_targets.lp"
heuristic_prg = root + "/encodings/heuristic.lp"
minimal_completion_wb_prg = (
    root + "/encodings/card_min_completions_all_targets_with_bounds.lp"
)
completion_prg = root + "/encodings/completions_all_targets.lp"


def get_unproducible(network, seeds, targets):
    network_f = utils.to_file(network)
    seed_f = utils.to_file(seeds)
    target_f = utils.to_file(targets)
    prg = [unproducible_prg, network_f, seed_f, target_f]
    options = ""
    best_model = None
    models = clyngor.solve(prg, options=options, use_clingo_module=False)
    for model in models.discard_quotes.by_arity:
        best_model = model
    os.unlink(network_f)
    os.unlink(seed_f)
    os.unlink(target_f)
    return best_model


def compute_ireactions(instance):
    instance_f = utils.to_file(instance)
    prg = [ireaction_prg, instance_f]
    best_model = None

    models = clyngor.solve(prg, use_clingo_module=False)
    for model in models.discard_quotes.by_arity:
        best_model = model
    os.unlink(instance_f)

    output = set()
    for pred in best_model:
        if pred == "ireaction":
            for a in best_model[pred]:
                output.add(Atom('ireaction("' + a[0] + '","' + a[1] + '")'))

    return TermSet(output)


def get_minimal_completion_size(draft, repairnet, seeds, targets):

    instance = TermSet(draft.union(repairnet).union(targets).union(seeds))
    ireactions = compute_ireactions(instance)
    instance = TermSet(instance.union(ireactions))
    instance_f = utils.to_file(instance)

    prg = [minimal_completion_prg, instance_f]

    co = "--configuration=jumpy --opt-strategy=usc,5"

    optimum = None
    models = clyngor.solve(prg, options=co, use_clingo_module=False)
    for model in models.discard_quotes.by_arity:
        optimum = model

    os.unlink(instance_f)
    return optimum


def get_intersection_of_optimal_completions(draft, repairnet, seeds, targets, optimum):

    instance = TermSet(draft.union(repairnet).union(targets).union(seeds))
    ireactions = compute_ireactions(instance)
    instance = TermSet(instance.union(ireactions))
    instance_f = utils.to_file(instance)

    prg = [minimal_completion_prg, instance_f]

    options = (
        "--configuration=jumpy --opt-strategy=usc,5 --enum-mode=cautious --opt-mode=optN,"
        + str(optimum)
    )
    best_model = None
    models = clyngor.solve(prg, options=options, use_clingo_module=False)
    for model in models.discard_quotes.by_arity:
        best_model = model

    os.unlink(instance_f)
    return best_model


def get_union_of_optimal_completions(draft, repairnet, seeds, targets, optimum):

    instance = TermSet(draft.union(repairnet).union(targets).union(seeds))
    ireactions = compute_ireactions(instance)
    instance = TermSet(instance.union(ireactions))
    instance_f = utils.to_file(instance)
    prg = [minimal_completion_prg, instance_f]

    options = (
        "--configuration jumpy --opt-strategy=usc,5 --enum-mode=brave --opt-mode=optN,"
        + str(optimum)
    )

    models = clyngor.solve(prg, options=options, use_clingo_module=False)
    best_model = None
    for model in models.discard_quotes.by_arity:
        best_model = model

    os.unlink(instance_f)
    return best_model


def get_optimal_completions(draft, repairnet, seeds, targets, optimum, nmodels=0):

    instance = TermSet(draft.union(repairnet).union(targets).union(seeds))
    ireactions = compute_ireactions(instance)
    instance = TermSet(instance.union(ireactions))
    instance_f = utils.to_file(instance)

    prg = [minimal_completion_prg, instance_f]

    options = "--configuration=handy --opt-strategy=usc,0 --opt-mode=optN,{0}".format(
        optimum
    )

    models = clyngor.solve(
        prg, options=options, nb_model=nmodels, use_clingo_module=False
    ).by_arity.discard_quotes
    opt_models = clyngor.opt_models_from_clyngor_answers(models)

    return opt_models


def get_intersection_of_completions(draft, repairnet, seeds, targets):

    instance = TermSet(draft.union(repairnet).union(targets).union(seeds))
    ireactions = compute_ireactions(instance)
    if len(ireactions) == 0:
        return {}
    else:
        instance = TermSet(instance.union(ireactions))
        instance_f = utils.to_file(instance)

        prg = [completion_prg, instance_f]
        options = "--enum-mode=cautious --opt-mode=ignore "

        best_model = None
        models = clyngor.solve(prg, options=options, use_clingo_module=False)
        for model in models.discard_quotes.by_arity:
            best_model = model

        os.unlink(instance_f)
        return best_model



#!python
# Copyright (c) 2012, Sven Thiele <sthiele78@gmail.com>
#
# This file is part of meneco.
#
# meneco is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# meneco is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with meneco.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import json
import logging
import sys

from clyngor.as_pyasp import Atom, TermSet

from meneco import query, sbml, utils

logger = logging.getLogger(__name__)


def cmd_meneco(argv):
    """Run meneco from shell
    """
    parser = argparse.ArgumentParser()

    parser.add_argument(
        "-d", "--draftnet", help="metabolic network in SBML format", required=True
    )
    parser.add_argument("-s", "--seeds", help="seeds in SBML format", required=True)
    parser.add_argument("-t", "--targets", help="targets in SBML format", required=True)

    parser.add_argument(
        "-r",
        "--repairnet",
        help="perform network completion using REPAIRNET "
        "a metabolic network in SBML format",
    )

    parser.add_argument(
        "--enumerate",
        help="enumerate all minimal completions",
        action="store_true",
        default=False,
    )
    parser.add_argument(
        "--json",
        help="produce JSON output",
        required=False,
        action="store_true",
        default=False,
    )

    args = parser.parse_args(argv)

    result = run_meneco(
        args.draftnet,
        args.seeds,
        args.targets,
        args.repairnet,
        args.enumerate,
        args.json,
    )
    if args.json:
        print(json.dumps(result, indent=4))


def extract_xreactions(model, return_atom=True):
    lst = set(a[0] for pred in model if pred == "xreaction" for a in model[pred])
    if return_atom:
        atom = TermSet(
            Atom('xreaction("' + a[0] + '","' + a[1] + '")')
            for pred in model
            if pred == "xreaction"
            for a in model[pred]
        )
        return lst, atom
    else:
        return lst


def extract_unprod_target(model):
    return set(
        a[0] for pred in model if pred == "unproducible_target" for a in model[pred]
    )


def run_meneco(
    draftnet: str,
    seeds: str,
    targets: str,
    repairnet: str,
    enumeration: bool,
    json_output: bool,
):
    """Complete metabolic network by selecting reactions from a database

    Args:
        draftnet: SBML file name of metabolic network
        seeds: SBML file name seeds
        targets: SBML file name of targets
        repairnet: SBML file name of repair database
    """
    result = {}

    logger.info("Reading draft network ...")
    result["Draft network file"] = draftnet
    if not json_output:
        print(f"Draft network file: {draftnet}")
    draftnet = sbml.readSBMLnetwork(draftnet, "draft")
    # draftnet.to_file("draftnet.lp")

    logger.info("Reading seeds ...")
    result["Seeds file"] = seeds
    if not json_output:
        print(f"Seeds file: {seeds}")
    seeds = sbml.readSBMLseeds(seeds)
    # seeds.to_file("seeds.lp")

    logger.info("Reading targets ...")
    result["Targets file"] = targets
    if not json_output:
        print("Targets file: {0}\n".format(targets))
    targets = sbml.readSBMLtargets(targets)
    # targets.to_file("targets.lp")

    logger.info("Checking draftnet for unproducible targets ...")
    model = query.get_unproducible(draftnet, seeds, targets)

    unproducible_targets_lst = list(extract_unprod_target(model))
    unproducible_targets_lst.sort()

    result["Unproducible targets"] = unproducible_targets_lst
    if not json_output:
        print(
            f"{len(unproducible_targets_lst)} unproducible targets"
        )
        if len(unproducible_targets_lst) > 0 :
            print("    {0}\n".format("\n    ".join(unproducible_targets_lst)))
        else: print()


    if len(unproducible_targets_lst) == 0:
        utils.clean_up()
        return result

    if repairnet is None:
        utils.clean_up()
        return result

    logger.info("Reading repair db ...")
    result["Repair db file"] = repairnet
    if not json_output:
        print(f"Repair db file: {repairnet}\n")
    repairnet = sbml.readSBMLnetwork(repairnet, "repairnet")
    # repairnet.to_file("repairnet.lp")

    all_reactions = draftnet
    all_reactions = TermSet(all_reactions.union(repairnet))
    logger.info("Checking draftnet + repairnet for unproducible targets ...")
    model = query.get_unproducible(all_reactions, seeds, targets)
    never_producible = list(extract_unprod_target(model))
    never_producible.sort()

    result["Unreconstructable targets"] = never_producible
    if not json_output:
        print(
            f"{len(never_producible)} unreconstructable targets"
        )
        if len(never_producible) > 0 :
            print("    {0}\n".format("\n    ".join(never_producible)))
        else: print()

    reconstructable_targets = set()
    reconstructable_targets_atoms = TermSet()
    for t in unproducible_targets_lst:
        if not (t in never_producible):
            reconstructable_targets.add(t)
            reconstructable_targets_atoms.add(Atom('target("' + t + '")'))

    reconstructable_targets = list(reconstructable_targets)
    reconstructable_targets.sort()
    result["Reconstructable targets"] = reconstructable_targets
    if not json_output:
        print(
            f"{len(reconstructable_targets)} reconstructable targets"
        )
        if len(reconstructable_targets) > 0 :
            print("    {0}\n".format("\n    ".join(reconstructable_targets)))
        else: print()

    if len(reconstructable_targets) == 0:
        utils.clean_up()
        return result

    essential_reactions = TermSet()
    essential_reactions_to_print = set()
    essential_reactions_target = {}
    for t in reconstructable_targets:
        single_target = TermSet()
        single_target.add(Atom('target("' + t + '")'))
        logger.info(f"Computing essential reactions for {t} ...")
        essentials = query.get_intersection_of_completions(
            draftnet, repairnet, seeds, single_target
        )

        essentials_to_print, essentials_atoms = extract_xreactions(essentials, True)
        essentials_to_print = list(essentials_to_print)
        essentials_to_print.sort()
        essential_reactions_target[t] = essentials_to_print
        if not json_output:
            print(
                f"{len(essentials_to_print)} essential reactions for target {t}"
            )
            if len(essentials_to_print) > 0 :
                print("    {0}\n".format("\n    ".join(essentials_to_print)))
            else: print()

        essential_reactions = TermSet(essential_reactions.union(essentials_atoms))
        essential_reactions_to_print = set(
            essential_reactions_to_print.union(essentials_to_print)
        )

    result["Essential reactions"] = essential_reactions_target
    if not json_output:
        print(
            f"Overall {len(essential_reactions_to_print)} essential reactions found"
        )
        if len(essential_reactions_to_print) > 0 :
            print("    {0}\n".format("\n    ".join(essential_reactions_to_print)))
        else: print()

    logger.info("Adding essential reactions to network!")
    draftnet = TermSet(draftnet.union(essential_reactions))

    utils.clean_up()

    # draftnet.to_file("draft.lp")
    # repairnet.to_file("repairnet.lp")
    # unproducible_targets.to_file("targets.lp")
    # seeds.to_file("seeds.lp")

    logger.info("Computing one minimal completion to produce all targets ...")
    one_min_sol = query.get_minimal_completion_size(
        draftnet, repairnet, seeds, reconstructable_targets_atoms
    )

    one_min_sol_lst = list(extract_xreactions(one_min_sol, False))
    one_min_sol_lst.sort()
    optimum = len(one_min_sol_lst)

    result["One minimal completion"] = one_min_sol_lst
    if not json_output:
        print(
            f"One minimal completion of size {len(one_min_sol_lst)}"
        )
        if len(one_min_sol_lst) > 0 :
            print("    {0}\n".format("\n    ".join(one_min_sol_lst)))
        else: print()

    logger.info(
        f"Computing common reactions in all completion with size {optimum} ..."
    )
    intersection_sol = query.get_intersection_of_optimal_completions(
        draftnet, repairnet, seeds, reconstructable_targets_atoms, optimum
    )

    intersection_sol_lst = list(extract_xreactions(intersection_sol, False))
    intersection_sol_lst.sort()
    result["Intersection of cardinality minimal completions"] = list(
        intersection_sol_lst
    )
    if not json_output:
        print("Intersection of cardinality minimal completions")
        if len(intersection_sol_lst) > 0 :
            print("    {0}\n".format("\n    ".join(intersection_sol_lst)))
        else: print("    is empty")

    logger.info(
        f"Computing union of reactions from all completion with size {optimum} ..."
    )
    union_sol = query.get_union_of_optimal_completions(
        draftnet, repairnet, seeds, reconstructable_targets_atoms, optimum
    )

    union_sol_lst = list(extract_xreactions(union_sol, False))
    union_sol_lst.sort()

    result["Union of cardinality minimal completions"] = union_sol_lst
    if not json_output:
        print("Union of cardinality minimal completions")
        if len(union_sol_lst) > 0 :
            print("    {0}\n".format("\n    ".join(union_sol_lst)))
        else: print("    is empty")

    if enumeration:
        logger.info(f"Computing all completions with size {optimum} ...")
        enumeration_sol = query.get_optimal_completions(
            draftnet, repairnet, seeds, reconstructable_targets_atoms, optimum
        )
        count = 1
        enumeration_sol_lst = []
        for model in enumeration_sol:
            model_lst = extract_xreactions(model, False)
            enumeration_sol_lst.append(list(model_lst))

            if not json_output:
                print(
                    "Completion {0}:\n    {1}\n".format(count, "\n    ".join(model_lst))
                )
            count += 1
        result["All cardinality minimal completions"] = enumeration_sol_lst

    return result


if __name__ == "__main__":
    cmd_meneco(sys.argv[1:])



# Copyright (c) 2014, Sven Thiele <sthiele78@gmail.com>
#
# This file is part of meneco.
#
# meneco is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# meneco is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with meneco.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import setup

setup(
    name="Meneco",
    version="2.0.2",
    url="http://bioasp.github.io/meneco/",
    license="GPLv3+",
    description="Metabolic Network Completion. Compute minimal completions "
    "to your draft network with reactions from a repair network.",
    long_description=open("README.md", encoding="utf8").read(),
    long_description_content_type="text/markdown",
    author="Sven Thiele",
    author_email="sthiele78@gmail.com",
    packages=["meneco"],
    package_dir={"meneco": "meneco"},
    package_data={"meneco": ["encodings/*.lp"]},
    # scripts          = ['meneco.py'],
    entry_points={"console_scripts": ["meneco = meneco.__main__:main_meneco"]},
    install_requires=["clyngor_with_clingo"],
)



# Copyright (c) 2014, Sven Thiele <sthiele78@gmail.com>
#
# This file is part of meneco.
#
# meneco is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# meneco is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with meneco.  If not, see <http://www.gnu.org/licenses/>.

import logging
import xml.etree.cElementTree as etree

from clyngor.as_pyasp import Atom, TermSet

logger = logging.getLogger(__name__)


def get_model(sbml):
    model_element = None
    for e in sbml:
        if e.tag[0] == "{":
            _uri, tag = e.tag[1:].split("}")
        else:
            tag = e.tag
        if tag == "model":
            model_element = e
            break
    return model_element


def get_listOfSpecies(model):
    listOfSpecies = None
    for e in model:
        if e.tag[0] == "{":
            _uri, tag = e.tag[1:].split("}")
        else:
            tag = e.tag
        if tag == "listOfSpecies":
            listOfSpecies = e
            break
    return listOfSpecies


def get_listOfReactions(model):
    listOfReactions = None
    for e in model:
        if e.tag[0] == "{":
            _uri, tag = e.tag[1:].split("}")
        else:
            tag = e.tag
        if tag == "listOfReactions":
            listOfReactions = e
            break
    return listOfReactions


def get_listOfReactants(reaction):
    listOfReactants = None
    for e in reaction:
        if e.tag[0] == "{":
            _uri, tag = e.tag[1:].split("}")
        else:
            tag = e.tag
        if tag == "listOfReactants":
            listOfReactants = e
            break
    return listOfReactants


def get_listOfProducts(reaction):
    listOfProducts = None
    for e in reaction:
        if e.tag[0] == "{":
            _uri, tag = e.tag[1:].split("}")
        else:
            tag = e.tag
        if tag == "listOfProducts":
            listOfProducts = e
            break
    return listOfProducts


def get_score(reaction):
    score = 0
    for e in reaction:
        if e.tag[0] == "{":
            _uri, tag = e.tag[1:].split("}")
        else:
            tag = e.tag
        if tag == "annotation":
            for s in e:
                if s.tag[0] == "{":
                    _uri, stag = s.tag[1:].split("}")
                else:
                    stag = s.tag
                if stag == "score":
                    score = s.attrib.get("val")
    return score


def readSBMLnetwork(filename, name):

    lpfacts = set()
    tree = etree.parse(filename)
    sbml = tree.getroot()
    model = get_model(sbml)
    listOfReactions = get_listOfReactions(model)

    for e in listOfReactions:
        if e.tag[0] == "{":
            _uri, tag = e.tag[1:].split("}")
        else:
            tag = e.tag
        if tag == "reaction":
            reactionId = e.attrib.get("id")
            lpfacts.add(Atom("reaction", ['"' + reactionId + '"', '"' + name + '"']))
            if e.attrib.get("reversible") == "true":
                lpfacts.add(
                    Atom("reversible", ['"' + reactionId + '"', '"' + name + '"'])
                )

            listOfReactants = get_listOfReactants(e)
            if listOfReactants == None:
                logger.warning("Warning: {0} listOfReactants=None".format(reactionId))
            else:
                for r in listOfReactants:
                    lpfacts.add(
                        Atom(
                            "reactant",
                            [
                                '"' + r.attrib.get("species") + '"',
                                '"' + reactionId + '"',
                                '"' + name + '"',
                            ],
                        )
                    )

            listOfProducts = get_listOfProducts(e)
            if listOfProducts == None:
                logger.warning("Warning: {0} listOfProducts=None".format(reactionId))
            else:
                for p in listOfProducts:
                    lpfacts.add(
                        Atom(
                            "product",
                            [
                                '"' + p.attrib.get("species") + '"',
                                '"' + reactionId + '"',
                                '"' + name + '"',
                            ],
                        )
                    )

    if name == "draft":
        lpfacts.add(Atom("draft", ['"' + name + '"']))

    return TermSet(lpfacts)


def readSBMLnetwork_with_score(filename, name):

    lpfacts = set()
    tree = etree.parse(filename)
    sbml = tree.getroot()
    model = get_model(sbml)
    listOfReactions = get_listOfReactions(model)

    for e in listOfReactions:
        if e.tag[0] == "{":
            _uri, tag = e.tag[1:].split("}")
        else:
            tag = e.tag
        if tag == "reaction":
            reactionId = e.attrib.get("id")
            lpfacts.add(Atom("reaction", ['"' + reactionId + '"', '"' + name + '"']))
            if e.attrib.get("reversible") == "true":
                lpfacts.add(
                    Atom("reversible", ['"' + reactionId + '"', '"' + name + '"'])
                )

            listOfReactants = get_listOfReactants(e)
            if listOfReactants == None:
                logger.warning("Warning: {0} listOfReactants=None".format(reactionId))
            else:
                for r in listOfReactants:
                    lpfacts.add(
                        Atom(
                            "reactant",
                            [
                                '"' + r.attrib.get("species") + '"',
                                '"' + reactionId + '"',
                                '"' + name + '"',
                            ],
                        )
                    )

            listOfProducts = get_listOfProducts(e)
            if listOfProducts == None:
                logger.warning("Warning: {0} listOfProducts=None".format(reactionId))
            else:
                for p in listOfProducts:
                    lpfacts.add(
                        Atom(
                            "product",
                            [
                                '"' + p.attrib.get("species") + '"',
                                '"' + reactionId + '"',
                                '"' + name + '"',
                            ],
                        )
                    )

            score = get_score(e)
            if score != 0:
                value = int(float(score) * 1000)
                lpfacts.add(Atom("value", ['"' + reactionId + '"', str(value)]))
            else:
                # print " no value for ",Reaction_ID
                lpfacts.add(Atom("value", ['"' + reactionId + '"', "0"]))

    if name == "draft":
        lpfacts.add(Atom("draft", ['"' + name + '"']))

    return TermSet(lpfacts)


def readSBMLtargets(filename):

    lpfacts = set()
    tree = etree.parse(filename)
    sbml = tree.getroot()
    model = get_model(sbml)
    listOfSpecies = get_listOfSpecies(model)

    for e in listOfSpecies:
        if e.tag[0] == "{":
            _uri, tag = e.tag[1:].split("}")
        else:
            tag = e.tag
        if tag == "species":
            lpfacts.add(Atom("target", ['"' + e.attrib.get("id") + '"']))
    return TermSet(lpfacts)


def readSBMLseeds(filename):
    lpfacts = set()
    tree = etree.parse(filename)
    sbml = tree.getroot()
    model = get_model(sbml)
    listOfSpecies = get_listOfSpecies(model)

    for e in listOfSpecies:
        if e.tag[0] == "{":
            _uri, tag = e.tag[1:].split("}")
        else:
            tag = e.tag
        if tag == "species":
            lpfacts.add(Atom("seed", ['"' + e.attrib.get("id") + '"']))
    return TermSet(lpfacts)
